	list p=12f675
;***********************************************************************
;*  	Pinbelegung
;*	----------------------------------	
;*	GP: 	0 < analog ein (PTC)
;*		1 -
;*		2 > Ausgang fr Alarmton
;*		3 -
;*		4 -
;*		5 > schalter out
;*	
;***********************************************************************
;
;sprut (zero) Bredendiek 08/2005
;http://www.sprut.de
;
; Lfterschalter, temperaturabhngig
;
; Temperaturmessung mit die Pin  GP0
; Schalttransistor fr Lfter an GP5
; Alarm-Piepser an GP2
;
; Prozessor 12F675 
;
; Prozessor-Takt 4 MHz intern
;
; PTC-Spannung:
;	Temp.	Spannung	ADC		x 64
;	-20C	1,0106 V	206 = 00 CE	33 80
;	  0C	1,1593 V	237 = 00 ED	36 40
;	+12C	1,25   V	256 = 01 00	40 00
;	+20C	1,3125 V	268 = 01 0C	43 00
;	+32.4C	1,40   V	288 = 01 20	48 00 +
;	+37.5C	1,44   V	296 = 01 28	4A 00 +
;	+40C	1,4678 V	300 = 01 2C	46 00
;	+50C	1,54   V	316 = 01 3C	4F 00 +
;	+100C	1,9290 V	395 = 01 86	62 C0
;
; ADC = 237 + Temp x 1,575
;
; Temp = (ADC-237) / 1,575

; Einschalttemperatur =	37,5C  f1 > 49h
; Ausschalttemperatur =	32,4C	f1 < 48h
; Alarmtemperatur = 	50,0C  f1 > 4Eh
;
;***********************************************************************
; Includedatei fr den 12F675 einbinden

	#include <P12f675.INC>

	ERRORLEVEL      -302    	;SUPPRESS BANK SELECTION MESSAGES

; Configuration festlegen:
; kein Power on Timer, kein Watchdog, int-Oscillator, kein Brown out
	__CONFIG	_MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _BODEN_OFF


;***********************************************************************
; Variablen festlegen  20h ... 5Fh 

;W_save		Equ	0x20	; auch 0xA0
;Status_save	equ	0x21	; registers for saving context
;PCLATH_save	equ	0x22

NrMessung	EQU	0x24	; zhlt die 64 Temperaturmessungen
Fehler		equ	0x25	; Fehlerregister fr Mathematik

count		equ	0x26	; universeller zhler
loops		equ	0x27	; timer fr wait
loops2		equ	0x28	; timer fr wait

;16 Bit Rechenregister
f0		equ	0x32	; 
f1		equ	0x33	; 
;16 Bit Rechenregister
xw0		equ	0x34	; 
xw1		equ	0x35	; 

Ton1		equ	0x38	; Einschalttemperatur
Toff1		equ	0x3A	; Ausschalttemperatur
Temp		equ	0x40	; Puffer fr Stringausgabe
Alarm1		equ	0x41	; berhitzungstemperatur


;***********************************************************************
	org	0x00

	goto	Init


;***********************************************************************
; Initialisierung
;
Init
	; IO-Pins
	bcf	STATUS, RP0		; Bank 0
	clrf	GPIO			; aus!
	movlw	0x07
	movwf	CMCON			; Comparator aus
	bsf	STATUS, RP0		; Bank 1
	bcf	TRISIO,2		; GP2 output fr Alarmton
	bcf	TRISIO,5		; GP5 output fr Schalttransitor
	bcf	STATUS, RP0		; Bank0

	; internen Taktgenerator kalibrieren
	bsf	STATUS, RP0		; Bank 1
	call	0x3FF
	movwf	OSCCAL			; 4-MHz-Kalibrierung
	bcf	STATUS, RP0		; Bank 0

	; Interrupt
	bcf	INTCON, GIE		; Int deaktiviert


; ADC initialisieren
	; ADC einschalten
	BSF	ADCON0, ADON	; ADON=1
	; ADC-Eingang AN0 auswhlen
	BCF	ADCON0, CHS1	; ADCHS1=0
	BCF	ADCON0, CHS0	; ADCHS0=0
	; ADC speed fr 1,25 ... 5 MHz einstellen
	BSF	STATUS,RP0	; Bank1
	clrf	ANSEL
	BSF	ANSEL, ADCS0	; ADCS0=1 Fosc/8
	; AN0,1 analog
	BSF	ANSEL, ANS0	; AN1=analog
	BCF	STATUS,RP0	; Bank0
	; Daten rechtsbndig
	BCF	ADCON0, VCFG	; RA2=digital
	BSF	ADCON0, ADFM	; ADFM=1 rechtsbndig


	;Lftereinschalttemperatur festlegen
	movlw	0x49		; Ton1   37,5C
	movwf	Ton1

	movlw	0x48		; Toff1  32,4C
	movwf	Toff1

	movlw	0x4E
	movwf	Alarm1		; Alarm1 50,0C


;***********************************************************************
;Main 

Mainloop
	; Messen
	call	Temperatur	; Messung der Temperatur am Sensor 1
	call	Luft1		; Lfter ein-/ausschalten
	call	Hot		; Test auf bertemperatur

	call	wait75ms
	goto	Mainloop

;***********************************************************************
; Temperatur messen und in f1,f0 ablegen (in C als Binr-Integer)
; bei negativer Temperatur wird das Negativ-Flag=1 gesetzt

Temperatur
	; Startwert fr korrektes Runden
	clrf	f1
	movlw	D'32'
	movwf	f0

	; 64 Messungen
	movlw	D'64'
	movwf	NrMessung
Messung
	call	UMessen1	; AN0 nach xw1,xw0
	call	Add16 		; 16-bit add: f = f + xw
	decfsz	NrMessung, f
	goto	Messung

; PTC-Spannung:
;	Temp.	Spannung	ADC		x 64
;	-20C	1,0106 V	206 = 00 CE	33 80
;	  0C	1,1593 V	237 = 00 ED	36 40
;	+12C	1,25   V	256 = 01 00	40 00
;	+20C	1,3125 V	268 = 01 0C	43 00
;	+32.4C	1,40   V	288 = 01 20	48 00 +
;	+37.5C	1,44   V	296 = 01 28	4A 00 +
;	+40C	1,4678 V	300 = 01 2C	46 00
;	+50C	1,54   V	316 = 01 3C	4F 00 +
;	+100C	1,9290 V	395 = 01 86	62 C0
;
; ADC = 237 + Temp x 1,575
; Temp = (ADC-237) / 1,575

; Einschalttemperatur =	37,5C  f1 > 49h
; Ausschalttemperatur =	32,4C	f1 < 48h
; Alarmtemperatur = 	50,0C  f1 > 4Eh
	return


;***********************************************************************
; Spannung wandeln nach xw1, xw0 
UMessen1
	BSF	ADCON0, 1	; ADC starten
loop
	BTFSC	ADCON0, 1	; ist der ADC fertig?
	GOTO	loop		; nein, weiter warten
	movfw	ADRESH		; obere  2 Bit auslesen
	movwf	xw1		; obere  2-Bit nach xw1
	bsf	STATUS,RP0	; Bank1
	movfw	ADRESL		; untere 8 Bit auslesen
	bcf	STATUS,RP0	; Bank0
	movwf	xw0		; untere 8-Bit nach xw0

	clrf	count		; Warten, damit der ADC sich erholen kann
warten
	DECFSZ	count, f
	goto	warten
	return


;***********************************************************************
;Warteroutinen verschiedener Lnge 

; 75 ms warten
wait75ms
	movlw	D'75'		; 75ms
	movwf	loops	
	goto	WAIT

;***********************************************************************
;Zeitverzgerung um loops * 1 ms
; 4 MHz externer Takt bedeutet 1 MHz interner Takt
; also dauert 1 ms genau 1000 Befehle
; 100 Schleifen a 10 Befehle sind 1000 Befehle = 1 ms

WAIT
top     movlw   .100           ; timing adjustment variable (1ms)
        movwf   loops2
top2    nop                    ; sit and wait
        nop
        nop
        nop
	nop
        nop
        nop
        decfsz  loops2, F      ; inner loops complete?
        goto    top2           ; no, go again
                               ;
        decfsz  loops, F       ; outer loops complete?
        goto    top            ; no, go again
        retlw   0              ; yes, return from subWAIT


;***********************************************************************
;+++Mathematik-Routinen 
;
;***********************************************************************
; 16 Bit Subtraktion, bei berlauf (neg. Ergebnis) ist C gesetzt
Sub16				; 16 bit f:=f-xw   calc=xw cnt=f
	clrf	Fehler		; extraflags lschen
	movf    xw0, w		; f0=f0-xw0
	subwf   f0, f
	btfsc   STATUS,C
	goto    Sub16A
	movlw   0x01		; borgen von f1
	subwf   f1, f
	btfss   STATUS,C
	bsf	Fehler, C	; unterlauf
Sub16A
	movf    xw1,w		; f1=f1-xw1
        subwf   f1    , f
        btfss   STATUS, C
 	bsf	Fehler, C	; Unterlauf
	bcf	STATUS, C
	btfsc	Fehler, C
	bsf	STATUS, C
	return


;***********************************************************************
;16 bit Adition, C-Flag bei berlauf gesetzt
Add16 				; 16-bit add: f = f + xw
	movf	xw0,W		; low byte
	addwf	f0,F 		; low byte add

	movf	xw1,W 		; next byte
	btfsc	STATUS,C 	; skip to simple add if C was reset
	incfsz	xw1,W 		; add C if it was set
	addwf	f1,F 		; high byte add if NZ
	
	return 			; finished



;***********************************************************************
;+++ Ansteuerung der Peripherie (Lfter, Piepser)
;
;***********************************************************************
; Lfter ein/aus-schalten

Luft1
	; prfen ob Temp<Toff
	MOVFW	Toff1
	subwf	f1, w		; w:=f-w = temp-Toff
	btfss	STATUS, C
	bcf	GPIO,5		; Lfter aus
	; prfen ob Temp>Ton
	MOVFW	Ton1
	subwf	f1, w		; w:=f-w = temp-Ton
	btfsc	STATUS, C
	bsf	GPIO,5		; Lfter an
	return


;***********************************************************************
; bertemperaturtest
; Alarmton an RA2 falls f0 > Alarm (50C)

Hot
	; prfen ob Temp<Alarm
	MOVFW	Alarm1
	subwf	f1, w		; w:=f-w = temp-Alarm
	btfss	STATUS, C
	return			; <50C
	; fr einen 2kHz-Ton sind 4000 Flanken ntig
	; bei 4MHz sind das 250 Zyklen zwischen 2 Flanken

	movlw	D'100'		; 100 Schleifen fr 50 ms Sound
	movwf	Temp
Hot1
	bsf	GPIO, 2
	movlw	.25		; timing adjustment variable (1/4ms)	
	movwf	loops2
	movlw	1
	movwf	loops
	call	top2		; 250us Wait
	bcf	GPIO, 2
	movlw	.25		; timing adjustment variable (1/4ms)	
	movwf	loops2
	movlw	1
	movwf	loops
	call	top2		; 250us Wait
	decfsz	Temp, f
	goto	Hot1
	return


;***********************************************************************

	end


